/*******************************************************************************
  MPLAB Harmony Application Source File
  
  Company:
    Microchip Technology Inc.
  
  File Name:
    app.c

  Summary:
    This file contains the source code for the MPLAB Harmony application.

  Description:
    This file contains the source code for the MPLAB Harmony application.  It 
    implements the logic of the application's state machine and it may call 
    API routines of other MPLAB Harmony modules in the system, such as drivers,
    system services, and middleware.  However, it does not call any of the
    system interfaces (such as the "Initialize" and "Tasks" functions) of any of
    the modules in the system or make any assumptions about when those functions
    are called.  That is the responsibility of the configuration-specific system
    files.
 *******************************************************************************/

// DOM-IGNORE-BEGIN
/*******************************************************************************
Copyright (c) 2013-2014 released Microchip Technology Inc.  All rights reserved.

Microchip licenses to you the right to use, modify, copy and distribute
Software only when embedded on a Microchip microcontroller or digital signal
controller that is integrated into your product or third party product
(pursuant to the sublicense terms in the accompanying license agreement).

You should refer to the license agreement accompanying this Software for
additional information regarding your rights and obligations.

SOFTWARE AND DOCUMENTATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND,
EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION, ANY WARRANTY OF
MERCHANTABILITY, TITLE, NON-INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE.
IN NO EVENT SHALL MICROCHIP OR ITS LICENSORS BE LIABLE OR OBLIGATED UNDER
CONTRACT, NEGLIGENCE, STRICT LIABILITY, CONTRIBUTION, BREACH OF WARRANTY, OR
OTHER LEGAL EQUITABLE THEORY ANY DIRECT OR INDIRECT DAMAGES OR EXPENSES
INCLUDING BUT NOT LIMITED TO ANY INCIDENTAL, SPECIAL, INDIRECT, PUNITIVE OR
CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA, COST OF PROCUREMENT OF
SUBSTITUTE GOODS, TECHNOLOGY, SERVICES, OR ANY CLAIMS BY THIRD PARTIES
(INCLUDING BUT NOT LIMITED TO ANY DEFENSE THEREOF), OR OTHER SIMILAR COSTS.
 *******************************************************************************/
// DOM-IGNORE-END


// *****************************************************************************
// *****************************************************************************
// Section: Included Files 
// *****************************************************************************
// *****************************************************************************

#include "app.h"
#include "UTFT.h"
#include "DefaultFonts.c"

// *****************************************************************************
// *****************************************************************************
// Section: Global Data Definitions
// *****************************************************************************
// *****************************************************************************

#define KVA0_TO_KVA1(v)	((void *) ((unsigned)(v) | 0x20000000))

// *****************************************************************************
/* Application Data

  Summary:
    Holds application data

  Description:
    This structure holds the application's data.

  Remarks:
    This structure should be initialized by the APP_Initialize function.
    
    Application strings and buffers are be defined outside this structure.
*/

APP_DATA appData;
extern int gamestatus;

/* Array in the program memory with starting address PROGRAM_FLASH_BASE_ADDRESS_VALUE to write the data */
const uint32_t dataFlash[128] __attribute__((address(PROGRAM_FLASH_BASE_ADDRESS_VALUE))) = {0};

 extern unsigned int highscore;
// *****************************************************************************
// *****************************************************************************
// Section: Application Callback Functions
// *****************************************************************************
// *****************************************************************************

/* TODO:  Add any necessary callback functions.
*/

// *****************************************************************************
// *****************************************************************************
// Section: Application Local Functions
// *****************************************************************************
// *****************************************************************************


/* TODO:  Add any necessary local functions.
*/


// *****************************************************************************
// *****************************************************************************
// Section: Application Initialization and State Machine Functions
// *****************************************************************************
// *****************************************************************************

/*******************************************************************************
  Function:
    void APP_Initialize ( void )

  Remarks:
    See prototype in app.h.
 */

void APP_Initialize ( void )
{
    /* Place the App state machine in its initial state. */
    appData.state = APP_STATE_INIT;

    
    /* TODO: Initialize your application's state machine and other
     * parameters.
     */
}


/******************************************************************************
  Function:
    void APP_Tasks ( void )

  Remarks:
    See prototype in app.h.
 */

void APP_Tasks ( void )
{

    /* Check the application's current state. */
    switch ( appData.state )
    {
        /* Application's initial state. */
        case APP_STATE_INIT:
        {
            bool appInitialized = true;
       
        
            if (appInitialized)
            {
            
                appData.state = APP_STATE_SERVICE_TASKS;
            }
            break;
        }

        case APP_STATE_SERVICE_TASKS:
        {
            uint32_t *p; //pointer for flash memory
      
            InitLCD(LANDSCAPE);
            setBackColor(0, 0, 0);
            setFont(FontData);
            
            DRV_TMR0_Start();//60分の1秒タイマー２開始
            
            DRV_OC0_Start();//サウンド出力開始　タイマー３を用いたOutput Compare
            PLIB_TMR_Period16BitSet(TMR_ID_3, 0x0000);
            DRV_TMR1_Start();//タイマー３開始       
 
            //ハイスコアーをプログラムフラッシュメモリから呼び出す。
            //ハイスコアーがゼロの場合プログラムフラッシュメモリが使われたことがないので、
            //同メモリを消去して書き込み可能とする。最初はハイスコアーを10としておき、
            //その数値を同メモリに書き込む
            p = (uint32_t *) KVA0_TO_KVA1(PROGRAM_FLASH_BASE_ADDRESS);
            if(*p==0) {
                NVMpageErase(PROGRAM_FLASH_BASE_ADDRESS);
                NVMwriteWord(PROGRAM_FLASH_BASE_ADDRESS, 10);
                highscore=10;
            }
            else   highscore=*p;//ハイスコアーがすでに書き込まれている場合には、それを読む
 
            while(1){
                initgame(); //ゲーム初期化
                do{
                    nextstage(); //次ステージへ
                    do{
                        wait60thsecwithsound(1); //60分の1秒ウェイトと効果音更新
                        clearchar(); //キャラクター表示消去
                        button(); //ボタン押下状態読み込み
                        fire(); //ミサイル発射処理
                        movecannon(); //自機移動
                        movealien(); //インベーダー移動
                        moveufo(); //UFO移動
                        movemissile(); //ミサイル移動
                        checkcollision(); //各種衝突チェック
                        putmissile(); //ミサイル表示
                        putaliens(); //インベーダー表示
                        putufo(); //UFO表示
                        putcannon(); //自機表示
        				printscore(); //得点表示
                        gamestatus=checkgame(); //ゲームステータス更新
                    } while(gamestatus==0);
                    //gamestatus 0:通常、1:次ステージへ、2:ゲームオーバー
                } while(gamestatus==1);
                gameover(); //ゲームオーバー
                if(highscore>*p){//ハイスコアーが更新された場合、プログラムフラッシュメモリに書き込む
                    NVMpageErase(PROGRAM_FLASH_BASE_ADDRESS);
                    NVMwriteWord(PROGRAM_FLASH_BASE_ADDRESS, highscore);
                }
            } 
            
            appData.state = APP_STATE_SERVICE_TASKS1;
            break;
        }
        
        case APP_STATE_SERVICE_TASKS1:
        {
                      
            break;
        }
     

        /* TODO: implement your application state machine.*/
        

        /* The default state should never be executed. */
        default:
        {
            /* TODO: Handle error in application's state machine. */
            break;
        }
    }
}

/*******************************************************************************
 End of File
 */
